/*
 * FASSTAVR.c, Version 3.1
 *
 * Copyright 2008 by Christian Persson. 
 *
 * ATTiny 25 as monoflop connecting the FASST T6EXP TX modul to any PPM signal.
 *
 * The adapter transmits 7 channels of the input signal. If the PPM signal carries 
 * less than 7 channels, the adapter amends neutral servo positions. A -100% signal 
 * is generated for channel 8, thus providing the channel 3 failsafe setting.
 *
 * PB0 (pin5) is output, PB1 (pin6) is power down button input, PB2 (pin7) is PPM signal input.
 *
 * Timer 1 indentifies the input sync pulse, timer 0 controls output signal timing.
 * Rev 3.1: PB5 pullup resistor, configure PB0 output within main loop 
 */


#include <stdint.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h> 

// F_CPU = 8000000 defined in makefile
// all durations in s*8
#define  Pulse420  51 
#define  Pulse440  54
#define  Pulse400  49
#define  MotorOff  70
#define  Neutral  137	

register uint8_t InSync asm ("r2");					// timer1 sync flag
register uint8_t MAXCH asm ("r4");					// number of input channels
register uint8_t INCH asm ("r5");					// input channel counter
register uint8_t OUTCH asm ("r6");					// output channel counter


// INPUT: timer1 compare interrupt
ISR(TIM1_COMPA_vect)			
{
	InSync = 1;										// identify sync pulse
}

// OUTPUT: Timer0 compare interrupt
ISR(TIM0_COMPA_vect)
{	
	PINB = 0x01;									// toggle output pin
	TCCR0B = 0;										// stop timer
}

int main (void)
{

	CLKPR = 0x80;									// enable set clock prescaler
	CLKPR = 0x00;									// set cpu clock prescaler = 1
	
	WDTCR = 0x09;									// watchdog enable 32ms

	DDRB = 0x01;									// PB0 output
	PORTB = 0x36;									// PB0 low, activate PB1, PB2, PB4, PB5 pullup resistors

	TIMSK = 0x50;									// OCIE1A, OCIE0A enable
	OCR1A = 200;									// sync pulse > 3.2ms
	TCCR1 = 0x08;									// start timer1 prescaler /128
	
	sei();
	
	// main loop
	while(1)	{
		
		DDRB = 0x01;								// PB0 output
		while ((PINB & 0x04) != 0 && (PINB & 0x01) == 0);// poll while input pin is high and output pin is low
		PORTB = 0x37;								// set output pin high

		if (InSync > 0) {							// sync pulse has occurred
			OUTCH = 0;
			MAXCH = INCH;
			INCH = 0;
		}
		if (OUTCH == 0) {
			if ((PINB & 0x02) != 0) {				// regular mode
				OCR0A = Pulse420;
			} else {								// power down mode
				OCR0A = Pulse440;
			}
		} else if (OUTCH == 1) {
			OCR0A = Pulse420;
		} else {
			OCR0A = Pulse400;
		}
		TCNT0 = 0;									// start timer0
		TCCR0B = 0x03;								// prescaler /64		
		OCR1A = 200;								// sync pulse > 3.2ms
		TCNT1 = 0;									// restart timer1
		InSync = 0;									// reset sync flag
		if ((PINB & 0x04) == 0) INCH++;				// count input start pulses
		OUTCH++;
//		if (MAXCH > 8) MAXCH = 8;					// max 8 input pulses are valid 
		if (MAXCH > 8) MAXCH = 9;					// max 8 input pulses are valid
		
		while ((PINB & 0x04) == 0 || (PINB & 0x01) != 0);// poll while input pin is low or output pin is high
		
		if (OUTCH < 9 && OUTCH >= MAXCH) {
			if (OUTCH < 8) {
				OCR0A = Neutral;					// fill in neutral signals up to channel 7 
			}
			else {
				OCR0A = MotorOff;					// add motor off signal on channel 8
			} 
			TCNT0 = 0;								// start timer0
			TCCR0B = 0x03;							// prescaler /64						
		}
		wdt_reset();								// Reset Watchdog
	}

	return(0);
}
